/*
 *     SocialLedge.com - Copyright (C) 2013
 *
 *     This file is part of free software framework for embedded processors.
 *     You can use it and/or distribute it as long as this copyright header
 *     remains unmodified.  The code is free for personal use and requires
 *     permission to use in a commercial product.
 *
 *      THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 *      OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 *      MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 *      I SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
 *      CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *     You can reach the author of this software at :
 *          p r e e t . w i k i @ g m a i l . c o m
 */

/**
 * @file
 * @brief This is the application entry point.
 *          FreeRTOS and stdio printf is pre-configured to use uart0_min.h before main() enters.
 *          @see L0_LowLevel/lpc_sys.h if you wish to override printf/scanf functions.
 *
 */
#include "tasks.hpp"
#include "io.hpp"
#include "task.h"
#include "queue.h"
#include "event_groups.h"
#include "examples/examples.hpp"
#include "stdio.h"
#include "stdlib.h"
#include "LPC17xx.h"
#include "printf_lib.h"
#include <FreeRTOS.h>
#include "spiADC.hpp"
#include "my_midi.hpp"

/**
 * The main() creates tasks or "threads".  See the documentation of scheduler_task class at scheduler_task.hpp
 * for details.  There is a very simple example towards the beginning of this class's declaration.
 *
 * @warning SPI #1 bus usage notes (interfaced to SD & Flash):
 *      - You can read/write files from multiple tasks because it automatically goes through SPI semaphore.
 *      - If you are going to use the SPI Bus in a FreeRTOS task, you need to use the API at L4_IO/fat/spi_sem.h
 *
 * @warning SPI #0 usage notes (Nordic wireless)
 *      - This bus is more tricky to use because if FreeRTOS is not running, the RIT interrupt may use the bus.
 *      - If FreeRTOS is running, then wireless task may use it.
 *        In either case, you should avoid using this bus or interfacing to external components because
 *        there is no semaphore configured for this bus and it should be used exclusively by nordic wireless.
 */

#include "command_handler.hpp"

CMD_HANDLER_FUNC(myCmdHandler)
{
    /* cmdParams is a str passed to you after user's command.
     * Commands are for hercules are "action rConsumer,
     * action sConsumer, action rProducer, action sProducer
     *
     * output is a CharDev class where the command came from, so
     * we can use this to output a reply message.
     * See "handlers.cpp" for more examples
     */

    /* return false will display command's help to the user */
    return true; /* return true if command was successful */
}

char curr_volume = INIT_VOLUME;
char curr_instr = INIT_INSTR;
bool    NOTE1 = false, NOTE2 = false, NOTE3 = false, NOTE4 = false,
        NOTE5 = false, NOTE6 = false, NOTE7 = false, OCTAVE = false;

class ADCTask : public scheduler_task
{
    public:
        ADCTask(uint8_t priority) :
            scheduler_task("ADCTask", 512*4, priority)
        {
            /* Nothing to init */
            QueueHandle_t my_queue = xQueueCreate(1, sizeof(note_t));
            addSharedObject(MIDI_QUEUE, my_queue);
//            QueueHandle_t octave_queue = xQueueCreate(1, sizeof(note_t));
//            addSharedObject(MIDI_QUEUE, my_queue);
        }

        bool init(void)
        {
            spiADC::SSP_Init(); //initializes SSP
            spiADC::control_init();
            //Set initial instrument bank/instrument
            midi_send_msg(SET_PARAMETERS, 0x00, 0x00);
            midi_send_msg(SET_INSTR, INIT_INSTR, 0);

            return true;
        }

        bool run(void *p)
        {
            //int i;
            //for ( i = 0; i < 10000; i++ );
            curr_volume = change_volume(curr_volume);
            curr_instr = change_instr(curr_instr);

            int ch0, ch1, ch2, ch3, ch4, ch5, ch6, ch7;
            note_t key = quiet;

            ch0 = spiADC::read_ADC(0x80);
            ch1 = spiADC::read_ADC(0x90);
            ch2 = spiADC::read_ADC(0xA0);
            ch3 = spiADC::read_ADC(0xB0);
            ch4 = spiADC::read_ADC(0xC0);
            ch5 = spiADC::read_ADC(0xD0);
            ch6 = spiADC::read_ADC(0xE0);
            ch7 = spiADC::read_ADC(0xF0);

            printf("\nCh0 is %d\n", ch0);
            if (ch0 > 650)
            {

                //xQueueSend(getSharedObject(MIDI_QUEUE), &key, portMAX_DELAY);
            }
                printf("\nCh1 is %x\n", ch1);
            if (ch1 < 835)
            {
                if (NOTE1 == false)
                {

                    key = note1;
                    xQueueSend(getSharedObject(MIDI_QUEUE), &key, portMAX_DELAY);
                    NOTE1 = true;
                }
            }
            else
            {
                midi_send_msg(NOTE_OFF, 0x3C, MAX_VELOCITY);
                NOTE1 = false;
            }

            printf("\nCh2 is %x\n", ch2);
            if (ch2 < 835)
            {
                if (NOTE2 == false)
                {
                    key = note2;
                    xQueueSend(getSharedObject(MIDI_QUEUE), &key, portMAX_DELAY);
                    NOTE2 = true;
                }
            }
            else
            {
                midi_send_msg(NOTE_OFF, 0x3E, MAX_VELOCITY);
                NOTE2 = false;

            }
            printf("\nCh3 is %x\n", ch3);
            if (ch3 < 835)
            {
                if (NOTE3 == false)
                {
                    key = note3;
                    xQueueSend(getSharedObject(MIDI_QUEUE), &key, portMAX_DELAY);
                    NOTE3 = true;
                }
            }
            else
            {
                midi_send_msg(NOTE_OFF, 0x40, MAX_VELOCITY);
                NOTE3 = false;
            }
            printf("\nCh4 is %x\n", ch4);
            if (ch4 < 835)
            {
                if (NOTE4 == false)
                {
                    key = note4;
                    xQueueSend(getSharedObject(MIDI_QUEUE), &key, portMAX_DELAY);
                    NOTE4 = true;
                }
            }
            else
            {
                midi_send_msg(NOTE_OFF, 0x42, MAX_VELOCITY);
                NOTE4 = false;
            }
            printf("\nCh5 is %x\n", ch5);
            if (ch5 < 835)
            {
                if (NOTE5 == false)
                {
                    key = note5;
                    xQueueSend(getSharedObject(MIDI_QUEUE), &key, portMAX_DELAY);
                    NOTE5 = true;
                }
            }
            else
            {
                midi_send_msg(NOTE_OFF, 0x44, MAX_VELOCITY);
                NOTE5 = false;
            }
            printf("\nCh6 is %x\n", ch6);
            if (ch6 < 835)
            {
                if (NOTE6 == false)
                {
                    key = note6;
                    xQueueSend(getSharedObject(MIDI_QUEUE), &key, portMAX_DELAY);
                    NOTE6 = true;
                }
            }
            else
            {
                midi_send_msg(NOTE_OFF, 0x46, MAX_VELOCITY);
                NOTE6 = false;
            }
            printf("\nCh7 is %x\n", ch7);
            if (ch7 < 835)
            {
                if (NOTE7 == false)
                {
                    key = note7;
                    xQueueSend(getSharedObject(MIDI_QUEUE), &key, portMAX_DELAY);
                    NOTE7 = true;
                }

            }
            else
            {
                midi_send_msg(NOTE_OFF, 0x48, MAX_VELOCITY);
                NOTE7 = false;
            }

//            vTaskDelay(1000);

            return true;

        }
};


int main(void)
{

    scheduler_add_task(new terminalTask(PRIORITY_HIGH));

    scheduler_add_task(new ADCTask(PRIORITY_MEDIUM));
    scheduler_add_task(new MIDITask(PRIORITY_HIGH));


    scheduler_start(); ///< This shouldn't return
    return -1;
}
